/* -*- Mode: C++; indent-tabs-mode: nil; tab-width: 4 -*-
 * -*- coding: utf-8 -*-
 *
 * Copyright (C) 2024 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "test.h"

ScreenControlTest::ScreenControlTest()
{
     m_screen = new Screen();    
}

ScreenControlTest::~ScreenControlTest()
{
    delete m_screen;
    m_screen = nullptr;
}

void ScreenControlTest::testSetPrimaryScreenBrightness()
{
    uint bright = 99;
//        QThread::msleep(1500);
    bool result = m_screen->SetPrimaryScreenBrightness(bright);
    QCOMPARE(bright, m_screen->GetPrimaryScreenBrightness());
}

//void ScreenControlTest::testGetPrimaryScreenBrightness()
//{
//    uint bright = 99;
//    bool result = m_screen->SetPrimaryScreenBrightness(bright);
//    QVERIFY(result);
//}

void ScreenControlTest::testIncreaseBuiltinScreenBrightness()
{
    int errorTimes = 0;
    if (Screen::eBrightnessControl::powerManager != m_screen->getBrightControlMode()) {
        QVERIFY(true);
        return;
    }

    uint initValue = m_screen->GetPrimaryScreenBrightness();
    m_screen->SetPrimaryScreenBrightness(10);

    for (int var = 0; var < 10; ++var) {
        m_screen->IncreaseBuiltinScreenBrightness();
        uint ret = m_screen->GetPrimaryScreenBrightness();
        if (ret != 10+5*(var+1)) {
            errorTimes++;
        }
    }
    m_screen->SetPrimaryScreenBrightness(initValue);
    QCOMPARE(errorTimes, 0);
}

void ScreenControlTest::testDecreaseBuiltinBrightness()
{
    int errorTimes = 0;
    if (Screen::eBrightnessControl::powerManager != m_screen->getBrightControlMode()) {
        QVERIFY(true);
        return;
    }

    uint initValue = m_screen->GetPrimaryScreenBrightness();

    for (int var = 0; var < 10; ++var) {
        m_screen->IncreaseBuiltinScreenBrightness();
        uint ret = m_screen->GetPrimaryScreenBrightness();
        if (ret != 10-5*(var+1)) {
            errorTimes++;
        }
    }
    m_screen->SetPrimaryScreenBrightness(initValue);
    QCOMPARE(errorTimes, 0);
}

//void ScreenControlTest::testGetScreenMode()
//{

//}

void ScreenControlTest::testSetScreenMode()
{
    int errorTimes = 0;
    QString initMode = m_screen->GetScreenMode();
    const QStringList modes = {"primaryScreenOnlyMode","mirrorScreenMode","extensionMode","secondaryScreenOnlyMode"};
    for(QString mode: modes) {
        m_screen->SetScreenMode(mode);
        QThread::sleep(2);
        QString currentMode = m_screen->GetScreenMode();
        qDebug()<<"-"<<mode<<"-"<<currentMode;
        if (mode != currentMode) {
            errorTimes++;
        }
    }
    m_screen->SetScreenMode(initMode);
    QCOMPARE(errorTimes, 0);
}

void ScreenControlTest::testSetScreensParam()
{
    int errorTimes = 0;
      const QStringList modes = {"primaryScreenOnlyMode","mirrorScreenMode","extensionMode","secondaryScreenOnlyMode"};
    QJsonDocument parser;
    QString screenParam = m_screen->GetScreensParam();
    QVariantList outputs = parser.fromJson(screenParam.toLatin1().data()).toVariant().toList();
    if (outputs.count()<2) {
        QVERIFY(true);
        return;
    }
    m_screen->SetScreenMode("mirrorScreenMode");
    QThread::sleep(2);
    QString cloneParam = m_screen->GetScreensParam();


    m_screen->SetScreenMode("extensionMode");
    QThread::sleep(2);
    m_screen->SetScreensParam(cloneParam);
    qDebug()<<cloneParam;
    QCOMPARE("mirrorScreenMode", m_screen->GetScreenMode());
    m_screen->SetScreensParam(screenParam);
}

void ScreenControlTest::testGetScreensParam()
{

}

void ScreenControlTest::testSetEyeCareEnabled()
{
    bool ret = false;
    bool initialValue = false;

    initialValue = m_screen->IsEyeCareEnabled();
    m_screen->SetEyeCareEnabled(ret);
    QCOMPARE(ret, m_screen->IsEyeCareEnabled());

    ret =!ret;

    m_screen->SetEyeCareEnabled(ret);
    QCOMPARE(ret, m_screen->IsEyeCareEnabled());
    m_screen->SetEyeCareEnabled(initialValue);
}

//bool ScreenControlTest::testIsEyeCareEnabled()
//{

//}

void ScreenControlTest::testSetNightModeEnable()
{
    bool ret = false;
    bool initialValue = false;

    initialValue = m_screen->IsNightModeEnable();
    m_screen->SetNightModeEnable(ret);
    QCOMPARE(ret, m_screen->IsNightModeEnable());

    ret =!ret;

    m_screen->SetNightModeEnable(ret);
    QCOMPARE(ret, m_screen->IsNightModeEnable());
    m_screen->SetNightModeEnable(initialValue);
}

//void ScreenControlTest::testIsNightModeEnable()
//{

//}

void ScreenControlTest::testSetScreenBrightness()
{
    OutputGammaInfoList outputsList = m_screen->GetAllScreenBrightness();
    uint testBrightness[] = {100,90,80,70,60,50,40,30,20,10};
    uint errorTimes = 0;

    for (auto &outputInfo: outputsList) {
        m_screen->SetScreenBrightness(outputInfo.outputName, 100);
    }

    for (auto setBrightness: testBrightness) {
        for (auto &outputInfo: outputsList) {
            m_screen->SetScreenBrightness(outputInfo.outputName, setBrightness);
            QThread::msleep(900);
        }

        outputsList = m_screen->GetAllScreenBrightness();
        for (auto outputInfo: outputsList) {
            if (outputInfo.brignthess != setBrightness) {
                errorTimes++;
                qWarning() << outputInfo.outputName << "fail set brightness to" << setBrightness;
            }
        }
    }


    for (auto &outputInfo: outputsList) {
        m_screen->SetScreenBrightness(outputInfo.outputName, 100);
    }
    QThread::msleep(3500);
    QCOMPARE(errorTimes, 0);
}

void ScreenControlTest::testSetScreensBrightness()
{
    if (Screen::eBrightnessControl::powerManager == m_screen->getBrightControlMode()) {
        QVERIFY(true);
    }
}

void ScreenControlTest::testSetAllScreenSameBrightness()
{
    if (Screen::eBrightnessControl::powerManager == m_screen->getBrightControlMode()) {
        QVERIFY(true);
    }
}

//void ScreenControlTest::testGetAllScreenBrightness()
//{

//}


QTEST_MAIN(ScreenControlTest)

#include "moc_test.cpp" 
