/* -*- Mode: C++; indent-tabs-mode: nil; tab-width: 4 -*-
 * -*- coding: utf-8 -*-
 *
 * Copyright (C) 2024 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "touchpad.h"

TouchpadManager::TouchpadManager(QObject *parent):
    QObject(parent)
{
    QDBusConnection::sessionBus().connect("org.ukui.SettingsDaemon",
                                          "/org/ukui/SettingsDaemon/InputManager",
                                          "org.ukui.SettingsDaemon.InputManager",
                                          "touchpadStateChanged",
                                         this, SLOT(onTouchpadStateChanged(bool)));
}

TouchpadManager::~TouchpadManager()
{

}

bool TouchpadManager::IsEnable()
{
   QDBusReply<bool> reply = callInputDbus("getTouchpadState");
   if (reply.isValid()) {
        return reply.value();
   }

   return false;
}

void TouchpadManager::Enable(bool state)
{
    callInputDbus("setTouchpadState", state);
}

void TouchpadManager::onTouchpadStateChanged(bool state)
{
    Q_EMIT EnableChanged(state);
}

template <typename...Args>
QDBusMessage TouchpadManager::callInputDbus(const QString &method, Args &&...args)
{
    QDBusInterface inputManagerInterface("org.ukui.SettingsDaemon",
                                  "/org/ukui/SettingsDaemon/InputManager",
                                  "org.ukui.SettingsDaemon.InputManager");
    return inputManagerInterface.call(method, std::forward<Args>(args)...);
}
